<?php
//set_site_transient( 'update_plugins', null );
/**
 * Allows plugins to use their own update API.
 *
 * @subpackage  classes
 * @package     gb-elementor
 * 
 * @author      WP Beaver World
 * @since       1.0
 */
class GBEPluginUpdater {
	private $api_url   = '';
	private $name      = '';
	private $slug      = '';

	/**
	 * Class constructor.
	 *
	 * @uses plugin_basename()
	 * @uses hook()
	 *
	 * @param   string    $_api_url       The URL pointing to the custom API endpoint.
	 * @param   string    $_plugin_file   Path to the plugin file.
	 * @param   float     $version        version of the plugin.   
	 * @return  void
	 */
	function __construct( $_api_url, $version )
	{  
		$this->api_url  = trailingslashit( $_api_url );
		$this->name     = plugin_basename( GBE_FILE );
		$this->slug     = basename( dirname( $this->name ) );
		$this->version  = $version;

		// Set up hooks.
		$this->init();

		add_action( 'admin_footer', array( $this, 'gbe_hide_message' ) );
		add_action( 'admin_init', array( $this, 'gbe_show_changelog' ) );
	} 

	/**
	 * Set up WordPress filters to hook into WP's update process.
	 *
	 * @uses add_filter()
	 *
	 * @return void
	 */
	public function init()
	{
		//* Plugin update actions
		add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'check_gbe_plugin_update' ) );
		add_filter( 'plugins_api', array( $this, 'gbe_plugin_api_call' ), 10, 3 );

		add_action( 'after_plugin_row_' . $this->name, array( $this, 'gbe_show_update_notification' ), 10, 2 );
	}

	/**
	 * Take over the update check
	 *
	 * @author  WP Beaver World
	 * @since   1.0
	 */ 
	function check_gbe_plugin_update( $checked_data )
	{
		global $wp_version;

		//Comment out these two lines during testing.
		if ( empty( $checked_data->checked ) )
			return $checked_data;

		if( empty( $checked_data->checked[$this->slug .'/'. $this->slug .'.php'] ) )
			$version = GBE_VERSION;
		else
			$version = $checked_data->checked[$this->slug .'/'. $this->slug .'.php'];

		$args = array(
			'slug'    => $this->slug,
			'version' => $version
		);

		$request_string = array(
			'body' => array(
				'action'  => 'basic_check', 
				'request' => serialize($args),
				'api-key' => md5(get_bloginfo('url'))
			),
			'user-agent' => 'WordPress/' . $wp_version . '; ' . get_bloginfo('url')
		);

		// Start checking for an update
		$raw_response = wp_remote_post( $this->api_url, $request_string );
		$response = '';

		if ( ! is_wp_error($raw_response) && ( $raw_response['response']['code'] == 200 ) )
		{
			$response = unserialize($raw_response['body']);

			if ( is_object( $response ) && ! empty( $response ) )
			{
				$checked_data->response[$this->slug .'/'. $this->slug .'.php'] = $response;
			}
		}

		return $checked_data;
	}

	/**
	 * Take over the Plugin info screen
	 *  
	 * @author  WP Beaver World
	 * @since   1.0
	 */ 
	function gbe_plugin_api_call( $def, $action, $args )
	{
		global $wp_version;

		if ( $action != 'plugin_information' ) {
		 return $def;
		}

		if ( ! isset( $args->slug ) || ( $args->slug != $this->slug ) )
			return false;

		//* Get the current version
		$plugin_info     = get_site_transient('update_plugins');
		$current_version = $plugin_info->checked[$this->slug .'/'. $this->slug .'.php'];
		$args->slug      = $this->slug;
		$args->version   = $current_version; 

		$request_string = array(
			'body' => array(
				'action'  => $action, 
				'request' => serialize($args),
				'api-key' => md5(get_bloginfo('url'))
			),
			'user-agent'     => 'WordPress/' . $wp_version . '; ' . get_bloginfo('url')
		);

		$request = wp_remote_post($this->api_url, $request_string);

		if ( is_wp_error( $request ) )
		{
			$res = new WP_Error('plugins_api_failed', __('An Unexpected HTTP Error occurred during the API request.&lt;/p&gt; &lt;p&gt;&lt;a href="?" onclick="document.location.reload(); return false;"&gt;Try again&lt;/a&gt', 'gb-elementor' ), $request->get_error_message());
		} else {
			$res = unserialize($request['body']);

			if ($res === false)
				$res = new WP_Error('plugins_api_failed', __( 'An unknown error occurred', 'gb-elementor' ), $request['body']);
		}

		return $res;
	}

	/**
	 * Show update message and download now button
	 *  
	 * @author  WP Beaver World
	 * @since   1.0
	 */
	function gbe_show_update_notification( $file, $plugin )
	{
		global $plugin_slug, $api_url, $wp_version;

		if( ! current_user_can( 'update_plugins' ) )
		{
			return;
		}

		if ( $this->name != $file )
		{
			return;
		}

		if( ! array_key_exists( 'new_version', $plugin ) )
			return;

		if ( version_compare( GBE_VERSION, $plugin['new_version'], '<' ) ) {

			// build a plugin list row, with update notification
			$wp_list_table = _get_list_table( 'WP_Plugins_List_Table' );
			echo '<tr id="' . $this->slug . '-update-row" class="plugin-update-tr active" data-plugin="' . $this->slug . '/' . $this->slug . '.php" data-slug="' . $this->slug . '"><td colspan="' . $wp_list_table->get_column_count() . '" class="plugin-update colspanchange"><div class="update-message">';

			$changelog_link = self_admin_url( 'index.php?gbe_action=view_plugin_changelog&plugin=' . $this->name . '&slug=' . $this->slug . '&TB_iframe=true&width=772&height=911' );
			 
			
			printf(
				__( 'There is a new version of %1$s available. <a target="_blank" class="thickbox" href="%2$s">View version %3$s details</a> or <a href="%4$s">update now</a>.', 'gb-elementor' ),
				esc_html( $plugin['Title'] ),
				esc_url( $changelog_link ),
				esc_html( $plugin['new_version'] ),
				esc_url( wp_nonce_url( self_admin_url( 'update.php?action=upgrade-plugin&plugin=' ) . $this->name, 'upgrade-plugin_' . $this->name ) )
			);

			echo '</div></td></tr>';
		}
	} 

	/**
	 * Display changelog info
	 *  
	 * @author  WP Beaver World
	 * @since   1.0
	 */
	function gbe_show_changelog()
	{
		if( empty( $_REQUEST['gbe_action'] ) || 'view_plugin_changelog' != $_REQUEST['gbe_action'] ) {
			return;
		}

		if( empty( $_REQUEST['plugin'] ) )
		{
			return;
		}

		if( empty( $_REQUEST['slug'] ) )
		{
			return;
		}

		if( ! current_user_can( 'update_plugins' ) )
		{
			wp_die( __( 'You do not have permission to install plugin updates', 'gb-elementor' ), __( 'Error', 'gb-elementor' ), array( 'response' => 403 ) );
		}

		$response = file_get_contents( 'http://pwdtechnology.com/changelog/gb-elementor/changelog.txt' );

		if( $response )
		{
			echo '<div style="background:#fff;padding:10px;">' . str_replace( "\n", "<br/>", $response ) . '</div>';
		}

		exit;
	}

	function gbe_hide_message()
	{
		echo '<style type="text/css">#'. $this->slug .'-update{ display: none;}</style>';
	}
}